@@ -4,7 +4,7 @@
 /* SUMMARY
 /*	show Postfix queue file contents
 /* SYNOPSIS
-/*	\fBpostcat\fR [\fB-bdehnoqv\fR] [\fB-c \fIconfig_dir\fR] [\fIfiles\fR...]
+/*	\fBpostcat\fR [\fB-bdefhnoqv\fR] [\fB-c \fIconfig_dir\fR] [\fIfiles\fR...]
 /* DESCRIPTION
 /*	The \fBpostcat\fR(1) command prints the contents of the
 /*	named \fIfiles\fR in human-readable form. The files are
@@ -33,6 +33,8 @@
 /*	Show message envelope content.
 /* .sp
 /*	This feature is available in Postfix 2.7 and later.
+/* .IP \fB-f\fR
+/*	Prepend the file name to each output line.
 /* .IP \fB-h\fR
 /*	Show message header content.  The \fB-h\fR option produces
 /*	output from the beginning of the message up to, but not
@@ -76,7 +78,7 @@
 /*	The default location of the Postfix main.cf and master.cf
 /*	configuration files.
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
-/*	The list of environment parameters that a privileged Postfix
+/*	The list of environment variables that a privileged Postfix
 /*	process will import from a non-Postfix parent process, or name=value
 /*	environment overrides.
 /* .IP "\fBqueue_directory (see 'postconf -d' output)\fR"
@@ -147,6 +149,7 @@
 #define PC_FLAG_PRINT_RTYPE_DEC	(1<<5)	/* print decimal record type */
 #define PC_FLAG_PRINT_RTYPE_SYM	(1<<6)	/* print symbolic record type */
 #define PC_FLAG_RAW		(1<<7)	/* don't follow pointers */
+#define PC_FLAG_PRINT_PATHNAME	(1<<8)	/* print pathname */
 
 #define PC_MASK_PRINT_TEXT	(PC_FLAG_PRINT_HEADER | PC_FLAG_PRINT_BODY)
 #define PC_MASK_PRINT_ALL	(PC_FLAG_PRINT_ENV | PC_MASK_PRINT_TEXT)
@@ -244,6 +247,8 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	 * changed.
 	 */
 #define PRINT_MARKER(flags, fp, offset, type, text) do { \
+    if ((flags) & PC_FLAG_PRINT_PATHNAME) \
+	vstream_printf("%s: ", VSTREAM_PATH(fp)); \
     if ((flags) & PC_FLAG_PRINT_OFFSET) \
 	vstream_printf("%9lu ", (unsigned long) (offset)); \
     if (flags & PC_FLAG_PRINT_RTYPE_DEC) \
@@ -253,6 +258,8 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 } while (0)
 
 #define PRINT_RECORD(flags, offset, type, value) do { \
+    if ((flags) & PC_FLAG_PRINT_PATHNAME) \
+	vstream_printf("%s: ", VSTREAM_PATH(fp)); \
     if ((flags) & PC_FLAG_PRINT_OFFSET) \
 	vstream_printf("%9lu ", (unsigned long) (offset)); \
     if (flags & PC_FLAG_PRINT_RTYPE_DEC) \
@@ -274,7 +281,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 		    break;
 		/* Optimization: skip to extracted segment marker. */
 		if (do_print == 0 && (flags & PC_FLAG_PRINT_ENV)
-		    && data_offset >= 0 && data_size >= 0
+		    && data_offset > 0 && data_size >= 0
 		&& vstream_fseek(fp, data_offset + data_size, SEEK_SET) < 0)
 		    msg_fatal("seek error: %m");
 	    }
@@ -289,7 +296,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 		PRINT_MARKER(flags, fp, offset, rec_type, "MESSAGE CONTENTS");
 	    /* Optimization: skip to extracted segment marker. */
 	    if ((flags & PC_MASK_PRINT_TEXT) == 0
-		&& data_offset >= 0 && data_size >= 0
+		&& data_offset > 0 && data_size >= 0
 		&& vstream_fseek(fp, data_offset + data_size, SEEK_SET) < 0)
 		msg_fatal("seek error: %m");
 	    /* Update the state machine, even when skipping. */
@@ -363,6 +370,8 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	 */
 	if (do_print == 0)
 	    continue;
+	if (flags & PC_FLAG_PRINT_PATHNAME)
+	    vstream_printf("%s: ", VSTREAM_PATH(fp));
 	if (flags & PC_FLAG_PRINT_OFFSET)
 	    vstream_printf("%9lu ", (unsigned long) offset);
 	if (flags & PC_FLAG_PRINT_RTYPE_DEC)
@@ -491,7 +500,7 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "bc:dehoqrs:v")) > 0) {
+    while ((ch = GETOPT(argc, argv, "bc:defhoqrs:v")) > 0) {
 	switch (ch) {
 	case 'b':
 	    flags |= PC_FLAG_PRINT_BODY;
@@ -503,6 +512,9 @@ int     main(int argc, char **argv)
 	case 'd':
 	    flags |= PC_FLAG_PRINT_RTYPE_DEC;
 	    break;
+	case 'f':
+	    flags |= PC_FLAG_PRINT_PATHNAME;
+	    break;
 	case 'e':
 	    flags |= PC_FLAG_PRINT_ENV;
 	    break;
