@@ -155,6 +155,9 @@
 #include <xtext.h>
 #include <uxtext.h>
 #include <smtputf8.h>
+#if defined(USE_TLS) && defined(USE_TLSRPT)
+#include <tlsrpt_wrapper.h>
+#endif
 
 /* Application-specific. */
 
@@ -475,6 +478,11 @@ int     smtp_helo(SMTP_STATE *state)
 		else
 		    session->features &= ~SMTP_FEATURE_ESMTP;
 	    }
+#ifdef USE_TLSRPT
+	    if (state->tlsrpt
+		&& (state->misc_flags & SMTP_MISC_FLAG_IN_STARTTLS) == 0)
+		trw_set_ehlo_resp(state->tlsrpt, resp->str);
+#endif
 	}
 	if ((session->features & SMTP_FEATURE_ESMTP) == 0) {
 	    where = "performing the HELO handshake";
@@ -484,6 +492,10 @@ int     smtp_helo(SMTP_STATE *state)
 				       "host %s refused to talk to me: %s",
 				       session->namaddr,
 				       translit(resp->str, "\n", " ")));
+#ifdef USE_TLSRPT
+	    if (state->tlsrpt)
+		trw_set_ehlo_resp(state->tlsrpt, resp->str);
+#endif
 	}
     } else {
 	where = "performing the LHLO handshake";
@@ -798,11 +810,19 @@ int     smtp_helo(SMTP_STATE *state)
 	     * although support for it was announced in the EHLO response.
 	     */
 	    session->features &= ~SMTP_FEATURE_STARTTLS;
-	    if (TLS_REQUIRED(state->tls->level))
+	    if (TLS_REQUIRED(state->tls->level)) {
+#ifdef USE_TLSRPT
+		if (state->tlsrpt)
+		    trw_report_failure(state->tlsrpt,
+				       TLSRPT_STARTTLS_NOT_SUPPORTED,
+				        /* additional_detail= */ (char *) 0,
+				        /* failure_reason= */ (char *) 0);
+#endif
 		return (smtp_site_fail(state, STR(iter->host), resp,
 		    "TLS is required, but host %s refused to start TLS: %s",
 				       session->namaddr,
 				       translit(resp->str, "\n", " ")));
+	    }
 	    /* Else try to continue in plain-text mode. */
 	}
 
@@ -815,6 +835,13 @@ int     smtp_helo(SMTP_STATE *state)
 	 */
 	if (TLS_REQUIRED(state->tls->level)) {
 	    if (!(session->features & SMTP_FEATURE_STARTTLS)) {
+#ifdef USE_TLSRPT
+		if (state->tlsrpt)
+		    trw_report_failure(state->tlsrpt,
+				       TLSRPT_STARTTLS_NOT_SUPPORTED,
+				        /* additional_detail= */ (char *) 0,
+				        /* failure_reason= */ (char *) 0);
+#endif
 		return (smtp_site_fail(state, DSN_BY_LOCAL_MTA,
 				       SMTP_RESP_FAKE(&fake, "4.7.4"),
 			  "TLS is required, but was not offered by host %s",
@@ -888,7 +915,7 @@ static int smtp_start_tls(SMTP_STATE *state)
      * either the transport name or the values of CAfile and CApath. We use
      * the transport name.
      * 
-     * XXX: We store only one session per lookup key. Ideally the the key maps
+     * XXX: We store only one session per lookup key. Ideally the key maps
      * 1-to-1 to a server TLS session cache. We use the IP address, port and
      * ehlo response name to build a lookup key that works for split caches
      * (that announce distinct names) behind a load balancer.
@@ -929,6 +956,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 	TLS_PROXY_CLIENT_START_PROPS(&start_props,
 				     timeout = var_smtp_starttls_tmout,
 				     tls_level = state->tls->level,
+				     enable_rpk = state->tls->enable_rpk,
 				     nexthop = session->tls_nexthop,
 				     host = STR(iter->host),
 				     namaddr = session->namaddrport,
@@ -941,6 +969,12 @@ static int smtp_start_tls(SMTP_STATE *state)
 				     = vstring_str(state->tls->exclusions),
 				     matchargv = state->tls->matchargv,
 				     mdalg = var_smtp_tls_fpt_dgst,
+#ifdef USE_TLSRPT
+				     tlsrpt = state->tlsrpt,
+#else
+				     tlsrpt = 0,
+#endif
+				     fail_type = 0,
 				     dane = state->tls->dane);
 
 	/*
@@ -1051,6 +1085,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 			     fd = -1,
 			     timeout = var_smtp_starttls_tmout,
 			     tls_level = state->tls->level,
+			     enable_rpk = state->tls->enable_rpk,
 			     nexthop = session->tls_nexthop,
 			     host = STR(iter->host),
 			     namaddr = session->namaddrport,
@@ -1063,6 +1098,12 @@ static int smtp_start_tls(SMTP_STATE *state)
 			     = vstring_str(state->tls->exclusions),
 			     matchargv = state->tls->matchargv,
 			     mdalg = var_smtp_tls_fpt_dgst,
+#ifdef USE_TLSRPT
+			     tlsrpt = state->tlsrpt,
+#else
+			     tlsrpt = 0,
+#endif
+			     fail_type = state->tls->ext_policy_failure,
 			     dane = state->tls->dane);
 
 	/*
@@ -1123,10 +1164,42 @@ static int smtp_start_tls(SMTP_STATE *state)
      * we must check that here, and not state->tls->level.
      */
     if (TLS_MUST_MATCH(session->tls_context->level))
-	if (!TLS_CERT_IS_MATCHED(session->tls_context))
+	if (!TLS_CERT_IS_MATCHED(session->tls_context)) {
+#ifdef USE_TLSRPT
+
+	    /*
+	     * Don't create a TLSRPT failure report here, if the TLS engine
+	     * already reported a more specific reason.
+	     */
+	    if (state->tlsrpt && session->tls_context->rpt_reported == 0) {
+		if (!TLS_CERT_IS_TRUSTED(session->tls_context)) {
+		    (void) trw_report_failure(state->tlsrpt,
+					      TLSRPT_CERTIFICATE_NOT_TRUSTED,
+				        /* additional_detail= */ (char *) 0,
+					  /* failure_reason= */ (char *) 0);
+		} else {
+		    (void) trw_report_failure(state->tlsrpt,
+					   TLSRPT_CERTIFICATE_HOST_MISMATCH,
+				        /* additional_detail= */ (char *) 0,
+					  /* failure_reason= */ (char *) 0);
+		}
+	    }
+#endif
 	    return (smtp_site_fail(state, DSN_BY_LOCAL_MTA,
 				   SMTP_RESP_FAKE(&fake, "4.7.5"),
 				   "Server certificate not verified"));
+	}
+
+    /*
+     * Create a TLSRPT success report only if the TLS engine has not reported
+     * a failure (For example, the TLS handshake may be successful, but the
+     * security level was downgraded from opportunistic or half "dane" to
+     * "encrypt").
+     */
+#ifdef USE_TLSRPT
+    if (state->tlsrpt && session->tls_context->rpt_reported == 0)
+	(void) trw_report_success(state->tlsrpt);
+#endif
 
     /*
      * At this point we have to re-negotiate the "EHLO" to reget the
@@ -1146,10 +1219,10 @@ static void smtp_hbc_logger(void *context, const char *action,
     const SMTP_STATE *state = (SMTP_STATE *) context;
 
     if (*text) {
-	msg_info("%s: %s: %s %.60s: %s",
+	msg_info("%s: %s: %s %.200s: %s",
 		 state->request->queue_id, action, where, content, text);
     } else {
-	msg_info("%s: %s: %s %.60s",
+	msg_info("%s: %s: %s %.200s",
 		 state->request->queue_id, action, where, content);
     }
 }
@@ -1187,6 +1260,18 @@ static void smtp_text_out(void *context, int rec_type,
 	    if (data_left > 0 || rec_type == REC_TYPE_CONT) {
 		smtp_fputc(' ', session->stream);
 		state->space_left -= 1;
+
+		/*
+		 * XXX This can insert a line break into the middle of a
+		 * multi-byte character (not necessarily UTF-8). Note that
+		 * multibyte characters can span queue file records, for
+		 * example if line_length_limit == smtp_line_length_limit.
+		 */
+		if (state->logged_line_length_limit == 0) {
+		    msg_info("%s: breaking line > %d bytes with <CR><LF>SPACE",
+			     state->request->queue_id, var_smtp_line_limit);
+		    state->logged_line_length_limit = 1;
+		}
 	    }
 	} else {
 	    if (rec_type == REC_TYPE_CONT) {
@@ -1926,7 +2011,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 		 * not clobber this non-zero value once it is set. The
 		 * variable need not survive longjmp() calls, since the only
 		 * setjmp() which does not return early is the one sets this
-		 * condition, subquent failures always return early.
+		 * condition, subsequent failures always return early.
 		 */
 #define LOST_CONNECTION_INSIDE_DATA (except == SMTP_ERR_EOF)
 
